@@ -6,25 +6,25 @@
 #
 # These data are the X-Y coordinates of 20 replications of writing
 # the script "fda".  The subject was Jim Ramsay.  Each replication
-# is represented by 1401 coordinate values.  The scripts have been 
+# is represented by 1401 coordinate values.  The scripts have been
 # extensively pre-processed.  They have been adjusted to a common
 # length that corresponds to 2.3 seconds or 2300 milliseconds, and
 # they have already been registered so that important features in
 # each script are aligned.
-# 
+#
 # This analysis is designed to illustrate techniques for working
 # with functional data having rather high frequency variation and
 # represented by thousands of data points per record.  Comments
 # along the way explain the choices of analysis that were made.
-# 
-# The final result of the analysis is a third order linear 
-# differential equation for each coordinate forced by a 
+#
+# The final result of the analysis is a third order linear
+# differential equation for each coordinate forced by a
 # constant and by time.  The equations are able to reconstruct
 # the scripts to a fairly high level of accuracy, and are also
 # able to accommodate a substantial amount of the variation in
-# the observed scripts across replications.  by contrast, a 
+# the observed scripts across replications.  by contrast, a
 # second order equation was found to be completely inadequate.
-# 
+#
 # An interesting suprise in the results is the role placed by
 # a 120 millisecond cycle such that sharp features such as cusps
 # correspond closely to this period.  This 110-120 msec cycle
@@ -32,10 +32,11 @@
 # movements, such as speech, juggling and so on.
 #  -----------------------------------------------------------------------
 
-#  Last modified 18 August, 2007 by Giles Hooker
-#  Previously modified 2 March 2007 by spencer Graves
-#  Previously  modified 20 March 2006
+#  Last modified 28 August 2012 by Jim Ramsay
 
+#  Attach the FDA functions
+
+library(fda)
 
 #  Input the data.  These 20 records have already been
 #  normalized to a common time interval of 2300 milliseconds
@@ -69,17 +70,17 @@ fdaarray = handwrit
 fdatime  <- seq(0, 2300, len=1401)
 fdarange <- c(0, 2300)
 
-#  The basis functions will be B-splines, with a spline 
+#  The basis functions will be B-splines, with a spline
 #  placed at each knot.  One may question whether so many
-#  basis functions are required, but this decision is found to 
-#  be  essential for stable derivative estimation up to the 
+#  basis functions are required, but this decision is found to
+#  be  essential for stable derivative estimation up to the
 #  third order at and near the boundaries.
 
 #  Order 7 was used to get a smooth third derivative, which
 #  requires penalizing the size of the 5th derivative, which
 #  in turn requires an order of at least 7.
-#  This implies norder + no. of interior knots = 1399 + 7 = 1406 
-#  basis functions.  
+#  This implies norder + no. of interior knots = 1399 + 7 = 1406
+#  basis functions.
 
 nbasis   <- 1406
 norder   <-    7
@@ -93,12 +94,12 @@ fdafd  <- fd(array(0, c(nbasis,20,2)), fdabasis)
 lambda <- 1e8
 fdaPar <- fdPar(fdafd, 5, lambda)
 
-#  set up the functional data structure 
-#  (required 1.5 mins on an IBM X31 notebook)
+#  set up the functional data structure
+#  (required 1.5 mins on a Lenovo X201 laptop)
 
 smoothList <- smooth.basis(fdatime, fdaarray, fdaPar)
-fdafd <- smoothList$fd 
-df    <- smoothList$df 
+fdafd <- smoothList$fd
+df    <- smoothList$df
 gcv   <- smoothList$gcv
 #  Add suitable names for the dimensions of the data.
 fdafd$fdnames[[1]] <- "Milliseconds"
@@ -127,8 +128,8 @@ fdameanfd  <- mean(fdafd)
 fdamat     <- eval.fd(fdatime, fdafd)
 fdameanmat <- apply(fdamat, c(1,3), mean)
 
-#  Set up motor control clock cycle times at every 
-#  119 milliseconds. 
+#  Set up motor control clock cycle times at every
+#  119 milliseconds.
 
 cycle  <- seq(0,2300,119)
 ncycle <- length(cycle)
@@ -139,10 +140,10 @@ fdamatcycle     <- eval.fd(cycle, fdafd)
 fdameanmatcycle <- apply(fdamatcycle,c(1,3),mean)
 
 #  Indices of cycle times corresponding to important features:
-#  -- the cusp in "f", 
-#  -- the the cusp in "d", 
-#  -- the first cusp in "a", 
-#  -- the rest after the first cusp in "a", and 
+#  -- the cusp in "f",
+#  -- the the cusp in "d",
+#  -- the first cusp in "a",
+#  -- the rest after the first cusp in "a", and
 #  -- the second cusp in "a".
 #  It is remarkable that these features correspond so closely
 #  with clock cycle times!
@@ -157,10 +158,10 @@ fdameanfeature <- fdameanmatcycle[featureindex,]
 
 par(mfrow=c(1,1), pty="s")
 plot(fdameanmat[,1], fdameanmat[,2], type="l", lwd=2,
-     xlab="Metres", ylab="Metres", 
+     xlab="Metres", ylab="Metres",
      xlim=c(-.040, .040), ylim=c(-.040, .040),
      main="Mean script")
-points(fdameanmatcycle[-featureindex,1], 
+points(fdameanmatcycle[-featureindex,1],
        fdameanmatcycle[-featureindex,2],        cex=1.2,col=2,lwd=4)
 points(fdameanfeature[,1],  fdameanfeature[,2], cex=1.2,col=3,lwd=4)
 
@@ -171,16 +172,16 @@ points(fdameanfeature[,1],  fdameanfeature[,2], cex=1.2,col=3,lwd=4)
 
 par(mfrow=c(1,1), pty="s",ask=TRUE)
 for (i in 1:20) {
-    plot(fdamat[,i,1], fdamat[,i,2], type="l", lwd=2,  
+    plot(fdamat[,i,1], fdamat[,i,2], type="l", lwd=2,
          xlab="Metres", ylab="Metres",
          xlim=c(-.040, .040), ylim=c(-.040, .040),
          main=paste("Script",i))
-    points(fdamatcycle[-featureindex,i,1],   
-          fdamatcycle[-featureindex,i,2],         cex=1.2,col=2,lwd=4) 
-    points(fdafeature[,i,1],    fdafeature[,i,2],  cex=1.2,col=3,lwd=4) 
-    lines( fdameanmat[,1],      fdameanmat[,2],      lty=4) 
-    points(fdameanmatcycle[-featureindex,1], 
-           fdameanmatcycle[-featureindex,2],         cex=1.2,col=2,lwd=4) 
+    points(fdamatcycle[-featureindex,i,1],
+          fdamatcycle[-featureindex,i,2],         cex=1.2,col=2,lwd=4)
+    points(fdafeature[,i,1],    fdafeature[,i,2],  cex=1.2,col=3,lwd=4)
+    lines( fdameanmat[,1],      fdameanmat[,2],      lty=4)
+    points(fdameanmatcycle[-featureindex,1],
+           fdameanmatcycle[-featureindex,2],         cex=1.2,col=2,lwd=4)
     points(fdameanfeature[,1],  fdameanfeature[,2],  cex=1.2,col=3,lwd=4)
 }
 
@@ -199,13 +200,13 @@ D3fdameanmat <- apply(D3fdamat, c(1,3), mean)
 #  metres per second per second.
 
 #  Cycle and feature times are plotted as vertical
-#  dashed lines, un-featured cycle times as red 
-#  dotted lines, and cycle times of features as 
+#  dashed lines, un-featured cycle times as red
+#  dotted lines, and cycle times of features as
 #  heavier magenta solid lines.
 
 par(mfrow=c(1,1), mar=c(5,5,4,2), pty="m",ask=TRUE)
 for (i in 1:20) {
-    matplot(fdatime, cbind(1e6*D2fdamat[,i,1],1e6*D2fdamat[,i,2]), 
+    matplot(fdatime, cbind(1e6*D2fdamat[,i,1],1e6*D2fdamat[,i,2]),
          type="l", lty=1, cex=1.2, col=c(2,4),
          xlim=c(0, 2300), ylim=c(-12, 12),
          xlab="Milliseconds", ylab="Meters/msec/msec",
@@ -213,17 +214,18 @@ for (i in 1:20) {
     abline(h=0, lty=2)
     plotrange <- c(-12,12)
     for (k in 1:length(cycle)) abline(v=cycle[k], lty=2)
-    for (j in 1:length(featureindex)) 
+    for (j in 1:length(featureindex))
 	     abline(v=cycle[featureindex[j]], lty=1)
     legend(1800,11.5, c("X", "Y"), lty=1, col=c(2,4))
 }
 
-#  Compute and plot the acceleration magnitudes,  
-#  also called the tangential accelerations.  
+#  Compute and plot the acceleration magnitudes,
+#  also called the tangential accelerations.
 
 D2mag  <- sqrt(D2fdamat[,,1]^2 + D2fdamat[,,2]^2)
 D2magmean <- apply(D2mag,1,mean)
 
+cexval <- 1.2
 par(mfrow=c(1,1), mar=c(5,5,4,2)+cexval+2, pty="m",ask=FALSE)
 matplot(fdatime, 1e6*D2mag, type="l", cex=1.2,
      xlab="Milliseconds", ylab="Metres/sec/sec",
@@ -231,7 +233,7 @@ matplot(fdatime, 1e6*D2mag, type="l", cex=1.2,
      main="Acceleration Magnitude")
 plotrange <- c(0,12)
 for (k in 1:length(cycle)) abline(v=cycle[k], lty=2)
-for (j in 1:length(featureindex)) 
+for (j in 1:length(featureindex))
 	abline(v=cycle[featureindex[j]], lty=1)
 
 #  Plot the mean acceleration magnitude as well as
@@ -245,7 +247,7 @@ plot(fdatime, 1e6*D2magmean, type="l", cex=1.2,
      main="Mean acceleration Magnitude")
 plotrange <- c(0,8)
 for (k in 1:length(cycle)) abline(v=cycle[k], lty=2)
-for (j in 1:length(featureindex)) 
+for (j in 1:length(featureindex))
 	abline(v=cycle[featureindex[j]], lty=1)
 
 #  Plot each individual acceleration magnitude, along
@@ -254,21 +256,21 @@ for (j in 1:length(featureindex))
 par(mfrow=c(1,1), mar=c(5,5,4,2), pty="m", ask=TRUE)
 plotrange <- c(0,12)
 for (i in 1:20) {
-    plot(fdatime, 1e6*D2mag[,i], type="l", cex=1.2, 
-         xlim=c(0,2300), ylim=c(0,12), 
+    plot(fdatime, 1e6*D2mag[,i], type="l", cex=1.2,
+         xlim=c(0,2300), ylim=c(0,12),
          xlab="Milliseconds", ylab="Metres/sec/sec",
          main=paste("Script ",i))
     lines(fdatime,  1e6*D2magmean, lty=3)
     for (k in 1:length(cycle)) abline(v=cycle[k], lty=2)
-    for (j in 1:length(featureindex)) 
+    for (j in 1:length(featureindex))
 	     abline(v=cycle[featureindex[j]], lty=1)
 }
 
 #  ------------------------------------------------------------
-#                 Principal Differential Analysis 
+#                 Principal Differential Analysis
 #  A third order equation forced by a constant and time is
 #          estimated for X and Y coordinates separately.
-#  Forcing with constant and time is required to allow for 
+#  Forcing with constant and time is required to allow for
 #  an arbitrary origin and the left-right motion in X.
 #  ------------------------------------------------------------
 
@@ -280,12 +282,12 @@ ufdlist <- vector("list", 2)
 #  constant forcing
 constbasis <- create.constant.basis(fdarange)
 constfd    <- fd(matrix(1,1,20), constbasis)
-ufdlist[[1]] <- constfd               
+ufdlist[[1]] <- constfd
 # time forcing
 linbasis   <- create.monomial.basis(fdarange, 2)
 lincoef    <- matrix(0,2,20)
 lincoef[2,] <- 1
-ufdlist[[2]] <- fd(lincoef, linbasis) 
+ufdlist[[2]] <- fd(lincoef, linbasis)
 
 #  set up the corresponding weight functions
 
@@ -334,7 +336,7 @@ bwtlist[[3]] <- bfdPar
 pdaList <- pda.fd(xfdlist, bwtlist, awtlist, ufdlist)
 
 bestwtlist <- pdaList$bwtlist
-aestwtlist <- pdaList$awtlist 
+aestwtlist <- pdaList$awtlist
 resfdlist  <- pdaList$resfdlist
 
 #  evaluate forcing functions
@@ -355,11 +357,11 @@ bwtlist[[2]] <- bfdPar
 bwtlist[[3]] <- bfdPar
 
 #  carry out principal differential analysis
-#  (this takes about 2 minutes on an IBM X31 Notebook)
+#  (this takes about 2 minutes on an Lenovo X201 laptop)
 
 pdaList <- pda.fd(xfdlist, bwtlist, awtlist, ufdlist, difeorder)
 bestwtlist <- pdaList$bwtlist
-aestwtlist <- pdaList$awtlist 
+aestwtlist <- pdaList$awtlist
 resfdlist  <- pdaList$resfdlist
 
 #  evaluate forcing functions
@@ -393,12 +395,12 @@ b2vecX   <- eval.fd(fdatime, b2fdX)
 b2meanX  <- mean(b2vecX)
 
 par(mfrow=c(1,1), pty="m")
-plot(fdatime, b2vecX, type="l", cex=1.2, 
+plot(fdatime, b2vecX, type="l", cex=1.2,
      xlim=c(0, 2300), ylim=c(0, 6e-3))
 abline(h=b2meanX, lty=3)
 plotrange <- c(0,6e-3)
 for (k in 1:length(cycle)) abline(v=cycle[k], lty=2)
-for (j in 1:length(featureindex)) 
+for (j in 1:length(featureindex))
 	abline(v=cycle[featureindex[j]], lty=1)
 
 #  display coefficients for forcing weight functions
@@ -425,7 +427,7 @@ plot(fdatime, 1e9*resmeanvec, type="l", cex=1.2, col=2,
      xlab="Milliseconds", ylab="Meters/sec/sec/sec")
 lines(fdatime, 1e9*D3fdameanmat[,1], lty=2, col=3)
 
-# Define a functional data object for the 
+# Define a functional data object for the
 #  three derivative weight functions
 
 wcoef1 <- bestwtlist[[1]]$fd$coefs
@@ -445,9 +447,9 @@ ystart[1,1] <-   fdameanmat[1,1]
 ystart[2,2] <- D1fdameanmat[1,1]
 ystart[3,3] <- D2fdameanmat[1,1]
 
-#  solve the equation 
+#  solve the equation
 
-#  (Solving the equations with the following error tolerance takes 20 minutes 
+#  (Solving the equations with the following error tolerance takes 20 minutes
 #   on an IBM X31 notebook, and about 5 seconds in Matlab.  Swtich to Matlab
 #   if you have a lot of this kind of work to do.)
 
@@ -497,15 +499,14 @@ fdamat <- eval.fd(fdatime, fdafd)
 zmat   <- cbind(fdatime-1150,umatx)
 for (i in index) {
     xhat <- fdamat[,i,1] - lsfit(zmat, fdamat[,i,1])$residual
-    matplot(fdatime, cbind(xhat, fdamat[,i,1]), 
+    matplot(fdatime, cbind(xhat, fdamat[,i,1]),
             type="l", lty=c(1,3), cex=1.2,
-            xlim=c(0, 2300), ylim=c(-0.04, 0.04),   
+            xlim=c(0, 2300), ylim=c(-0.04, 0.04),
             main=paste("X curve ",i))
 }
 
 #  ------------------------------------------------------------
-#  analysis for Y: a third order equation forced by a 
-#  constant and time
+#                   Analysis for coordinate Y
 #  ------------------------------------------------------------
 
 #  Define the variable
@@ -529,7 +530,7 @@ bwtlist[[3]] <- bfdPar
 
 pdaList <- pda.fd(yfdlist, bwtlist, awtlist, ufdlist, difeorder)
 bestwtlist <- pdaList$bwtlist
-aestwtlist <- pdaList$awtlist 
+aestwtlist <- pdaList$awtlist
 resfdlist  <- pdaList$resfdlist
 
 #  evaluate forcing functions
@@ -554,7 +555,7 @@ bwtlist[[3]] <- bfdPar
 
 pdaList <- pda.fd(yfdlist, bwtlist, awtlist, ufdlist, difeorder)
 bestwtlist <- pdaList$bwtlist
-aestwtlist <- pdaList$awtlist 
+aestwtlist <- pdaList$awtlist
 resfdlist  <- pdaList$resfdlist
 
 #  evaluate forcing functions
@@ -588,12 +589,12 @@ b2vecY   <- eval.fd(fdatime, b2fdY)
 b2meanY  <- mean(b2vecY)
 
 par(mfrow=c(1,1), pty="m", ask=FALSE)
-plot(fdatime, b2vecY, type="l", cex=1.2, 
+plot(fdatime, b2vecY, type="l", cex=1.2,
      xlim=c(0, 2300), ylim=c(0, 6e-3))
 abline(h=b2meanY, lty=3)
 plotrange <- c(0,6e-3)
 for (k in 1:length(cycle)) abline(v=cycle[k], lty=2)
-for (j in 1:length(featureindex)) 
+for (j in 1:length(featureindex))
 	abline(v=cycle[featureindex[j]], lty=1)
 
 #  display coefficients for forcing weight functions
@@ -620,7 +621,7 @@ plot(fdatime, 1e9*resmeanvec, type="l", cex=1.2, col=2,
      xlab="Milliseconds", ylab="Meters/sec/sec/sec")
 lines(fdatime, 1e9*D3fdameanmat[,1], lty=2, col=3)
 
-# Define a functional data object for the 
+# Define a functional data object for the
 #  three derivative weight functions
 
 wcoef1 <- bestwtlist[[1]]$fd$coefs
@@ -640,14 +641,14 @@ ystart[1,1] <-   fdameanmat[1,2]
 ystart[2,2] <- D1fdameanmat[1,2]
 ystart[3,3] <- D2fdameanmat[1,2]
 
-#  solve the equation 
+#  solve the equation
 
-#  (Solving the equations with the following error tolerance takes 20 minutes 
+#  (Solving the equations with the following error tolerance takes 20 minutes
 #   on an IBM X31 notebook, and about 5 seconds in Matlab.  Swtich to Matlab
 #   if you have a lot of this kind of work to do.)
 
 EPSval = 1e-4
-odeList <- odesolv(bestwtlist, ystart, EPS=EPSval, MAXSTP=1e6)ystart
+odeList <- odesolv(bestwtlist, ystart, EPS=EPSval, MAXSTP=1e6)
 tY <- odeList[[1]]
 yY <- odeList[[2]]
 
@@ -694,29 +695,12 @@ fdamat[,,2] <- eval.fd(fdatime, fdafdY)
 zmat   <- cbind(fdatime-1150,umaty)
 for (i in index) {
     yhat <- fdamat[,i,2] - lsfit(zmat, fdamat[,i,2])$residual
-    matplot(fdatime, cbind(yhat, fdamat[,i,2]), 
-            type="l", lty=c(1,3), cex=1.2, 
-            xlim=c(0, 2300), ylim=c(-0.04, 0.04),   
+    matplot(fdatime, cbind(yhat, fdamat[,i,2]),
+            type="l", lty=c(1,3), cex=1.2,
+            xlim=c(0, 2300), ylim=c(-0.04, 0.04),
             main=paste("Y curve ",i))
 }
 
-#  -------------  Look at fit to script  -------------------
-
-#  plot fit to each script  hit any key after each plot
-
-par(mfrow=c(1,1), ask=TRUE, pty="m")
-index   <- 1:20
-zmatx   <- cbind(matrix(1,1401,1),umatx)
-zmaty   <- cbind(matrix(1,1401,1),umaty)
-for (i in index) {
-    xhat <- fdamat[,i,1] - lsfit(umatx, fdamat[,i,1])$residual
-    yhat <- fdamat[,i,2] - lsfit(umaty, fdamat[,i,2])$residual
-    plot(xhat, yhat, type="l", cex=cexval,
-         xlim=c(-0.05,0.05), ylim=c(-0.05,0.05),
-         xlab="X", ylab="Y", main=paste("Script ",i))
-    lines(fdamat[,i,1], fdamat[,i,2], lty=3)
-}
-
 #  plot the two weight functions for the second derivative
 
 par(mfrow=c(1,1), mar=c(5,5,4,2)+cexval+2, pty="m")
@@ -726,18 +710,7 @@ abline(h=b2meanX, lty=3, col=2)
 abline(h=b2meanY, lty=3, col=4)
 plotrange <- c(0,6e-3)
 for (k in 1:length(cycle)) abline(v=cycle[k], lty=2)
-for (j in 1:length(featureindex)) 
+for (j in 1:length(featureindex))
 	abline(v=cycle[featureindex[j]], lty=1)
 
-#  Conclusions:
-
-#  For each coordinate, a single differential equation was
-#  estimated to describe all 20 coordinate records 
-#  simulataneously.
-#  The differential equation solutions are readable.
-#  Moreover, they accommodate a good deal of the curve-to-curve
-#  variation in the scripts.  
-#  The cusp first in the "a" is not handled as well as the other cusps.
-#  This may be due to poor registration at this point, or to the fact
-#  that this cusp has a lot of variation from one replication to another.
 
